/**********************************************************************
** This program generates a distribution of total synaptic input
** for a series of pulses on a single synapse.
**           Copyright (C) 2007 Upinder S. Bhalla. and NCBS
** It is made available under the terms of the
** GNU Lesser General Public License version 2.1,
** which permits free distribution without warranty of any kind.
** See the file COPYING.LIB for the full notice.
**********************************************************************/
#include <iostream>
#include <fstream>
#include <vector>
#include <math.h>
#include <cassert>
#include "randnum.h"

using namespace std;

static const unsigned int NUM_CYCLES = 30000;

/*
 * This program generates a distribution of total synaptic input
 * for a series of pulses on a single synapse.
 * The program assumes that the probability of release
 * goes as p0 for the first pulse.
 * For all subsequent pulses, we have p1_no_prev if there has not been any
 * release so far, and p1_yes_prev if there has. 
 * It also assumes that the individual pulse contributions are the same.
 */

int main( int argc, char** argv )
{
	if ( argc < 5 ) {
		cout << "usage: " << argv[0] << " numPulses p0 p1_no_prev p1_yes_prev \n";
		return 0;
	}
	unsigned int numPulses = atoi( argv[1] );
	double p0 = atof( argv[2] );
	double p1_no_prev = atof( argv[3] );
	double p1_yes_prev = atof( argv[4] );
	double sum = 0.0;
	double sumsq = 0.0;

	mtseed( 0 ); // Set up an initial random seed.
	for ( unsigned int j = 0; j < NUM_CYCLES; j++ ) {
		double total = 0.0;
		bool yes_prev = 0;
		if ( numPulses > 0 && mtrand() < p0 ) {
			total += 1.0;
			yes_prev = 1;
		}
		for (unsigned int i = 1; i < numPulses; i++ ) {
			if ( yes_prev ) {
				if ( mtrand() < p1_yes_prev )
					total += 1.0;
			} else {
				if ( mtrand() < p1_no_prev ) {
					total += 1.0;
					yes_prev = 1;
				}
			}
		}
		sum += total;
		sumsq += total * total;
	}
	cout << numPulses << "	" << p0 << "	" << p1_no_prev << "	" <<
		p1_yes_prev << "	" << sum / NUM_CYCLES << "	"	<<
		sqrt ( ( sumsq - sum * sum / NUM_CYCLES ) / NUM_CYCLES ) << endl;
}
